<img style="float: right;" width="120px" src="images/MiniclipLogo.jpg"/>

# MCGoliath

MCGoliath is a module that allows applications written in C++ to interact with Miniclip's Goliath server.

## Goliath Events

This Goliath Events API was generated for the game **`agario_mobile`**.

### MCGoliath Initialization

MCGoliath must be initialized with the API Key and the HMAC Key (Shared Secret). The API Key and HMAC Key are different for each one of the environments available for the game: staging and production:

* Staging
    * API Key: `19141ad5-70d6-4411-9254-69452d57801f`
    * HMAC Key: `398512c2-1126-4339-aed5-e980002a84d8`
* Production
    * API Key: `20ee26e9-d831-4651-a384-8caba6dd2eba`
    * HMAC Key: `ec46567a-c197-47be-b1bb-f11cce4fc72f`

The code snippet below is an example on how to initialize MCGoliath for **`agario_mobile`** using the staging environment properties:

```cpp
// API Key and Shared Secret for the staging environment.
const auto& stagingEnvironmentProperties = mc::goliath::events::getEnvironmentProperties("staging");
const std::string apiKey       = stagingEnvironmentProperties.apiKey;
const std::string sharedSecret = stagingEnvironmentProperties.hmacKey;

// Initialize MCGoliath.
MCGoliathWrapper::initialize(apiKey, sharedSecret);
```

Before queueing events, it's necessary to set the user and session ids using their corresponding methods of the module:

```cpp
MCGoliathWrapper& goliath = MCGoliathWrapper::sharedGoliath();

goliath.setUserID("the user id");
goliath.setSessionID("the session id");
```

The user and the session ids can be changed anytime if necessary using the methods above.

After initialization the module can be used to queue events to the Goliath server. To queue events use the classes documented in the Goliath Events section below.

### Public API

The Goliath events API provide an easy interface to send events over to Goliath, allowing the developer to build the events, fill-in their fields, and post them when they're ready, without having to assemble a valid JSON object for each event.

Each event has properties that the developer can set, and a `queue` method that will send it to the Goliath server using MCGoliath. Before the event is sent, some fields will be filled automatically, it will be validated and messages will be logged with the problems that are found, i.e. required fields that haven't been set, and values too big to fit the allowed space on the server. These problems do not prevent the event from being sent.

All these Goliath Events exist inside the `mc::goliath::events` namespace, and are provided int the `GoliathEvents.hpp` and `GoliathEvents.cpp` files which must be added to your project to be compiled along with your game.

Each event has a corresponding class in the `mc::goliath::events` namespace.

To send an event, create an instance of the appropriated class, and set its properties to the appropriate values. Some properties are required, failure to set them will generate a log message but will *not* prevent the event from being sent to the Goliath server. Likewise, some properties have a maximum size, if that size is exceeded a log message is generated but it will *not* prevent the event from being sent.

When the event instance is populated, call its `queue()` method to queue it for sending.

#### **ClientInit** event

When a connection to the server is created by means of a login request (part of MCGoliath)

```cpp
class ClientInit {
public:
    void setDeviceCompromised(const bool& value);
    void setDob(const mc::Date& value);
    void setGender(const std::string& value);
    void setInstalledApps(const std::vector<mc::Value>& value);
    void setNotificationsEnabled(const bool& value);
    void setPushToken(const std::string& value);

    // Queues the event to be sent to the Goliath server.
    void queue();
}
```

##### DeviceCompromised field

If the device is compromised, i.e., jailbroken or rooted

Details:

* Type `bool`
* Optional

##### Dob field

Date of birth, if provided by the login platform in the format yyyy-mm-dd

Details:

* Type `mc::Date`
* Optional

##### Gender field

User gender, if provided by the login platform

Details:

* Type `std::string`
* Maximum size is 10 characters.
* Optional

##### InstalledApps field

* Type `std::vector<mc::Value>`
* Optional

##### NotificationsEnabled field

If the player has the option to receive push notifications enabled

Details:

* Type `bool`
* Optional

##### PushToken field

The token to be able to send push notifications. apns token in the case of iOS, gcm token for Android

Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

#### **ClientNotification** event

When the app is opened (or reopened) by touching a notification

```cpp
class ClientNotification {
public:
    void setNotificationId(const std::string& value);
    void setNotificationOrigin(const std::string& value);

    // Queues the event to be sent to the Goliath server.
    void queue();
}
```

##### NotificationId field

An identifier for the notification used to launch the game, if any

Details:

* Type `std::string`
* Maximum size is 256 characters.
* Required

##### NotificationOrigin field

The origin of the notification used to launch the game

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

#### **CustomSkin** event

Whenever a user enters the Skin Editor, leaves the Skin Editor or saves (buys) a custom skin

```cpp
class CustomSkin {
public:
    void setAction(const std::string& value);
    void setEditionId(const std::string& value);
    void setEntryPoint(const std::string& value);
    void setPaymentMethod(const std::string& value);
    void setSavedBackgroundColour(const std::string& value);
    void setSavedBorderColour(const std::string& value);

    // Queues the event to be sent to the Goliath server.
    void queue();
}
```

##### Action field

The action performed in the skin editor:
* enter_editor,
* leave_editor, or
* save_skin

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

##### EditionId field

Unique ID generated when the player enters the editor and sent with every action until leaving the editor.
Even if this spans over several game sessions. 

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

##### EntryPoint field

The entry point to the skin editor. It only applies when action is enter_editor. It can be one of the following:
* premium_tab, or
* owned_tab

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Optional

##### PaymentMethod field

The payment method used to acquire the custom skin. It only applies when action is save_skin.
* ticket, or
* currency 

Details:

* Type `std::string`
* Maximum size is 10 characters.
* Optional

##### SavedBackgroundColour field

The final colour of the background. It only applies when action is save_skin.

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Optional

##### SavedBorderColour field

The final colour of the border. It only applies when action is save_skin.

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Optional

#### **DropdownMessageClick** event

Whenever a user clicks on a button inside a dropdown message

```cpp
class DropdownMessageClick {
public:
    void setButtonType(const std::string& value);

    // Queues the event to be sent to the Goliath server.
    void queue();
}
```

##### ButtonType field

The type of the button pressed by the user

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

#### **ExternalIdUpdate** event

To be sent when the user in question associates a new external account (whether for the first time or after having disassociated that same account in the past).
This event should not be sent when the user associates a miniclip login.

```cpp
class ExternalIdUpdate {
public:
    void setId(const std::string& value);
    void setType(const std::string& value);

    // Queues the event to be sent to the Goliath server.
    void queue();
}
```

##### Id field

The id of the external account being connected.

Details:

* Type `std::string`
* Maximum size is 256 characters.
* Required

##### Type field

The type of external account being connected. It can be one of the following:

* apple
* google_gamer
* facebook
* facebook_for_business

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

#### **ExtraInit** event

Connection to the server is created by means of a login request

```cpp
class ExtraInit {
public:
    void setGameLanguage(const std::string& value);

    // Queues the event to be sent to the Goliath server.
    void queue();
}
```

##### GameLanguage field

The language used in-game

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

#### **InterstitialBannerFunnel** event


```cpp
class InterstitialBannerFunnel {
public:
    void setAdUnitId(const std::string& value);
    void setAuctionType(const std::string& value);
    void setFailureReason(const std::string& value);
    void setIntentionalClick(const bool& value);
    void setMediator(const std::string& value);
    void setNetwork(const std::string& value);
    void setPlacementName(const std::string& value);
    void setRequestId(const std::string& value);
    void setStep(const std::string& value);
    void setTechVersion(const std::string& value);
    void setType(const std::string& value);
    void setValue(const double& value);

    // Queues the event to be sent to the Goliath server.
    void queue();
}
```

##### AdUnitId field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### AuctionType field

waterfall/bidding

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Optional

##### FailureReason field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### IntentionalClick field

Undefined

Details:

* Type `bool`
* Optional

##### Mediator field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Required

##### Network field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### PlacementName field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### RequestId field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Required

##### Step field


Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

##### TechVersion field

Undefined

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

##### Type field


Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

##### Value field

price floor/bid

Details:

* Type `double`
* Optional

#### **OfferwallFunnel** event


```cpp
class OfferwallFunnel {
public:
    void setFailureReason(const std::string& value);
    void setIntentionalClick(const bool& value);
    void setMediator(const std::string& value);
    void setNetwork(const std::string& value);
    void setRequestId(const std::string& value);
    void setRewardAmount(const long& value);
    void setRewardProduct(const std::string& value);
    void setStep(const std::string& value);
    void setTechVersion(const std::string& value);

    // Queues the event to be sent to the Goliath server.
    void queue();
}
```

##### FailureReason field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### IntentionalClick field

Undefined

Details:

* Type `bool`
* Optional

##### Mediator field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Required

##### Network field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### RequestId field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Required

##### RewardAmount field


Details:

* Type `long`
* Optional

##### RewardProduct field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### Step field


Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

##### TechVersion field

Undefined

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

#### **PrivacyPolicyConsent** event

When:
* the user interacts with the Privacy Policy With Ad Consent popup
* the user interacts with the Advertising Consent popup after trying to watch a video or visit the offerwall
* the user interacts with the Advertising Consent popup after trying to r

```cpp
class PrivacyPolicyConsent {
public:
    void setClickResult(const std::string& value);
    void setContext(const std::string& value);
    void setStep(const std::string& value);

    // Queues the event to be sent to the Goliath server.
    void queue();
}
```

##### ClickResult field

It describes the result of the click performed. It can be either:
* privacy_policy (when the user clicks in the privacy policy link)
* gave_ad_permission (when the user ends up giving permission to targeted ads)
* denied_ad_permission (when the user ends 

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Optional

##### Context field

The context in which the step was performed. It can be one of the following:
* privacy_policy_with_ad_consent
* video_ad (when the step was performed when interacting with the Advertising Consent popup triggered by a video ad)
* offerwall (when the step w

Details:

* Type `std::string`
* Maximum size is 256 characters.
* Required

##### Step field

It can be one of the following:
* impression (when the user is exposed to one of the popups)
* click (when the user interacts with one of the buttons in one of the popups or when he selects allow in the settings menu)

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

#### **PromoFeatureClick** event


```cpp
class PromoFeatureClick {
public:
    void setActionId(const std::string& value);
    void setActionTarget(const std::string& value);
    void setActionType(const std::string& value);
    void setCampaignId(const std::string& value);
    void setGroupId(const std::string& value);
    void setPopupGeneration(const std::string& value);
    void setPopupInteraction(const std::string& value);
    void setPromoId(const std::string& value);

    // Queues the event to be sent to the Goliath server.
    void queue();
}
```

##### ActionId field

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### ActionTarget field

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### ActionType field

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### CampaignId field

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### GroupId field

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### PopupGeneration field

* Type `std::string`
* Maximum size is 50 characters.
* Required

##### PopupInteraction field

* Type `std::string`
* Maximum size is 50 characters.
* Required

##### PromoId field

* Type `std::string`
* Maximum size is 256 characters.
* Required

#### **PromoFeatureImpression** event


```cpp
class PromoFeatureImpression {
public:
    void setCampaignId(const std::string& value);
    void setGroupId(const std::string& value);
    void setPopupGeneration(const std::string& value);
    void setPromoId(const std::string& value);

    // Queues the event to be sent to the Goliath server.
    void queue();
}
```

##### CampaignId field

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### GroupId field

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### PopupGeneration field

* Type `std::string`
* Maximum size is 50 characters.
* Required

##### PromoId field

* Type `std::string`
* Maximum size is 256 characters.
* Required

#### **PromoPromotionClick** event

Click table for promotion popup

```cpp
class PromoPromotionClick {
public:
    void setActionId(const std::string& value);
    void setActionTarget(const std::string& value);
    void setActionType(const std::string& value);
    void setCampaignId(const std::string& value);
    void setGroupId(const std::string& value);
    void setPopupGeneration(const std::string& value);
    void setPopupInteraction(const std::string& value);
    void setPromoId(const std::string& value);

    // Queues the event to be sent to the Goliath server.
    void queue();
}
```

##### ActionId field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### ActionTarget field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### ActionType field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### CampaignId field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### GroupId field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### PopupGeneration field


Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

##### PopupInteraction field


Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

##### PromoId field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Required

#### **PromoPromotionImpression** event

Impression table for promo popup

```cpp
class PromoPromotionImpression {
public:
    void setCampaignId(const std::string& value);
    void setGroupId(const std::string& value);
    void setPopupGeneration(const std::string& value);
    void setPromoId(const std::string& value);

    // Queues the event to be sent to the Goliath server.
    void queue();
}
```

##### CampaignId field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### GroupId field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### PopupGeneration field


Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

##### PromoId field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Required

#### **Share** event

When a sharing button is shown

```cpp
class Share {
public:
    void setContext(const std::string& value);
    void setExposureResult(const std::string& value);
    void setSocialPlatform(const std::string& value);

    // Queues the event to be sent to the Goliath server.
    void queue();
}
```

##### Context field

The in-game event that was shared: match_results, level_up, skin_bought, level_up_veteran, etc.

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

##### ExposureResult field

The reaction to the sharing button: create_post or dismiss_popup.

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

##### SocialPlatform field

The social platform used to share the accomplishment: facebook, twitter or googleplus.

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

#### **SubscriptionPopupInteraction** event

When the user interacts with the weekly subscription popup

```cpp
class SubscriptionPopupInteraction {
public:
    void setAction(const std::string& value);
    void setOrigin(const std::string& value);

    // Queues the event to be sent to the Goliath server.
    void queue();
}
```

##### Action field

The action performed.

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

##### Origin field

The entry point where the interaction started.

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

#### **VideoAdFunnel** event


```cpp
class VideoAdFunnel {
public:
    void setAdUnitId(const std::string& value);
    void setAuctionType(const std::string& value);
    void setFailureReason(const std::string& value);
    void setIntentionalClick(const bool& value);
    void setMediator(const std::string& value);
    void setNetwork(const std::string& value);
    void setPlacementName(const std::string& value);
    void setRequestId(const std::string& value);
    void setRewardAmount(const long& value);
    void setRewardProduct(const std::string& value);
    void setStep(const std::string& value);
    void setTechVersion(const std::string& value);
    void setValue(const double& value);

    // Queues the event to be sent to the Goliath server.
    void queue();
}
```

##### AdUnitId field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### AuctionType field

waterfall/bidding

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Optional

##### FailureReason field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### IntentionalClick field

Undefined

Details:

* Type `bool`
* Optional

##### Mediator field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Required

##### Network field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### PlacementName field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### RequestId field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Required

##### RewardAmount field


Details:

* Type `long`
* Optional

##### RewardProduct field


Details:

* Type `std::string`
* Maximum size is 256 characters.
* Optional

##### Step field


Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

##### TechVersion field

Undefined

Details:

* Type `std::string`
* Maximum size is 50 characters.
* Required

##### Value field

price floor/bid

Details:

* Type `double`
* Optional

## Goliath Events Origin

These Goliath Events were generated for the game **`agario_mobile`**, id **884**. The Schema version that matches this SDK is **v2.16.0**, and the server response version is **2**. Please make sure with the B.I. team that you’re using the appropriate versions to your current configuration.